// Control functions for pitch, roll, yaw, and throttle
#include "Arduino.h"
#include "Flux.hpp"
#include "Eeprom.h"

extern FluxEeprom eeprom;

// Pulse widths as received by remote control 1000 - 2000
extern volatile int rcThrottle, rcRoll, rcPitch, rcYaw;

// State variables
extern float anglePitch, angleRoll; // the current angles in degrees
extern float gyroYaw; // the current angle velocity in degrees per second
extern float cosAlpha; // Cosine of QC inclination

// -----------------------------------------------------------------------------------
int throttleControl() {
  // Compensate force loss due to copter inclination
  int pwThrottle = 1000 + (rcThrottle - 1000) / cosAlpha;
  return pwThrottle;
}
// -----------------------------------------------------------------------------------
const int MAX_PITCH_ROLL_ERROR_SUM = 5000;
static const float gainPitchRoll = 24.0 / 500.0; // 24 degrees on either side if stick is in full excursion
const float DECAY_SECONDS = 4.0; // error sums should decay in 4 seconds to 1/e to compensate for drift
const float DECAY_FACTOR = exp( - STEP_TIME / DECAY_SECONDS);

// Caution: Keep pitchControl() in sync with rollControl() below!
int pitchControl() { // Input: rcPitch, anglePitch
  static float pitchErrorSum;  // intermediate storage for the integral part of PID controller
  static float pitchLastError; // intermediate storage for the differential part of PID controller
  float pitchSetDeg = (1500 - rcPitch) * gainPitchRoll;
  float pitchError = pitchSetDeg - anglePitch;
  pitchErrorSum += pitchError;
  pitchErrorSum = constrain(pitchErrorSum, -MAX_PITCH_ROLL_ERROR_SUM, MAX_PITCH_ROLL_ERROR_SUM) * DECAY_FACTOR;
  if (rcThrottle < THROTTLE_SAFE) pitchErrorSum = 0;  // Assure decent start-up
  // Here comes the PID formula:
  float pwPitch = eeprom.pPitchRoll * pitchError                       // p
                  + eeprom.iPitchRoll * pitchErrorSum                  // i
                  + eeprom.dPitchRoll * (pitchError - pitchLastError); // d
  pitchLastError = pitchError;
  return pwPitch;
}
// -----------------------------------------------------------------------------------
// Caution: Keep rollControl() in sync with pitchControl above!
int  rollControl() { // Input: rcRoll, angleRoll
  static float rollErrorSum;  // intermediate storage for the integral part of PID controller
  static float rollLastError; // intermediate storage for the differential part of PID controller
  float rollSetDeg = (rcRoll - 1500) * gainPitchRoll;
  float rollError = rollSetDeg - angleRoll;
  rollErrorSum += rollError;
  rollErrorSum = constrain(rollErrorSum, -MAX_PITCH_ROLL_ERROR_SUM, MAX_PITCH_ROLL_ERROR_SUM) * DECAY_FACTOR;
  if (rcThrottle < THROTTLE_SAFE) rollErrorSum = 0;  // Assure decent start-up
  // Here comes the PID formula:
  float pwRoll = eeprom.pPitchRoll * rollError                      // p
                 + eeprom.iPitchRoll * rollErrorSum                 // i
                 + eeprom.dPitchRoll * (rollError - rollLastError); // d
  rollLastError = rollError;
  return pwRoll;
}
// -----------------------------------------------------------------------------------
int yawControl() {
  // Note that the sign of the RC yaw as well as the gyro signal is negative with respect to the
  // reference coordinate system and orientation (turn clockwise with increasing axis value)
  const float gainYaw = 90.0 / 500.0; // Full stick excursion is 90°/s
  int yawError = (rcYaw - 1500) *  gainYaw - gyroYaw;
  static float yawErrorSum;
  if (rcThrottle < THROTTLE_SAFE) yawErrorSum = 0;  // Assure decent start-up
  yawErrorSum += yawError;
  yawErrorSum = constrain(yawErrorSum, -200.0, 200.0);
  float pwYaw = eeprom.pYaw * yawError + eeprom.iYaw * yawErrorSum;
  return pwYaw;
}
