// Flux main program
#include "Flux.hpp"

extern void readEeprom();
extern bool isEepromChecksumCorrect();
extern Tune service();
extern void setupImu();
extern void readImu();
extern void calibrateGyros();
extern void calculateState();
extern void controlEscs(Tune, unsigned long);
extern void measureVibration();
extern float getVibration();

const int LED_PIN = 12;
const int LOW_PIN = A0;
const int SERVICE_MODE_IF_LOW = A1;

// State variables
extern float anglePitch, angleRoll, angleYaw; // the current angles in degrees
extern float gyroPitch, gyroRoll, gyroYaw; // the current angle velocities in degrees per second

const unsigned long loopPeriod = 1000 * LOOP_PERIOD; // unit is microsecond;
static Tune tune; // operating mode, regular flight is Tune::OFF
//==========================================================================================================
void setup() {
  // Configure output pins
  pinMode(4, OUTPUT); // Motor 1
  pinMode(5, OUTPUT); // Motor 2
  pinMode(6, OUTPUT); // Motor 3
  pinMode(7, OUTPUT); // Motor 4
  pinMode(LED_PIN, OUTPUT);
  pinMode(0, OUTPUT); // Testpin

  readEeprom();
  setupImu();
  calibrateGyros();

  // Configure hardware interrupts for remote control PWM measurements:
  PCICR  |= (1 << PCIE0);   // Set PCIE0 to enable PCMSK0 scan.
  PCMSK0 |= (1 << PCINT0);  // Set PCINT0 (digital input 8) to trigger an interrupt on state change.
  PCMSK0 |= (1 << PCINT1);  // Set PCINT1 (digital input 9) to trigger an interrupt on state change.
  PCMSK0 |= (1 << PCINT2);  // Set PCINT2 (digital input 10)to trigger an interrupt on state change.
  PCMSK0 |= (1 << PCINT3);  // Set PCINT3 (digital input 11)to trigger an interrupt on state change.

  pinMode(LOW_PIN, OUTPUT);
  digitalWrite(LOW_PIN, LOW);
  pinMode(SERVICE_MODE_IF_LOW, INPUT_PULLUP);
 
  if (digitalRead(SERVICE_MODE_IF_LOW) == LOW) {
    tune = service();
  } else {
    tune = Tune::OFF; // regular operation
  }

  if (!isEepromChecksumCorrect()) {
    while (true) { // EEPROM corrupted, blink slowly and block further execution.
      digitalWrite(LED_PIN, HIGH);
      delay(500);
      digitalWrite(LED_PIN, LOW);
      delay(500);
    }
  }
}
//==========================================================================================================
void loop() {
  static int loopCount;
  static enum { DISABLED, OFF, ON } ledBlink = ON;
  PORTD |= B00000001; // Testpin up

  unsigned long loopStartTime = micros(); // Microseconds from start. Rolls over after 2^32 us = 1.193 hours
  controlEscs(tune, loopStartTime);
  readImu();
  
  if (tune == Tune::OFF) { // regular operation
    calculateState();
    if (++loopCount == 32) { // after 128 ms
      loopCount = 0;
      switch (ledBlink) {
        case DISABLED:
          digitalWrite(LED_PIN, LOW);   // Turn off LED forever
          break;
        case ON:
          digitalWrite(LED_PIN, LOW);   // Turn off LED
          ledBlink = OFF;
          break;
        case OFF:
          digitalWrite(LED_PIN, HIGH);  // Turn on LED
          ledBlink = ON;
          break;
      }
    }
  }

  switch (tune) { // tuning functions
    case Tune::BALANCE_PROP_FR:
    case Tune::BALANCE_PROP_RR:
    case Tune::BALANCE_PROP_RL:
    case Tune::BALANCE_PROP_FL:
      measureVibration();
      if (++loopCount > 125) {
        Serial.println((int) (getVibration() / 10000.0));
        loopCount = 0;
      }
      break;
    default:
      break;
  }

  PORTD &= B11111110; // Testpin down
  if (micros() - loopStartTime > (loopPeriod + 50)) ledBlink = DISABLED;  // Turn off LED if loop time exceeds loopPeriod once.
  while (micros() < loopStartTime + loopPeriod) ;  // Wait until cycle has passed.
}

