#include "Arduino.h"
#include "eeprom.h"
#include "Flux.hpp"

extern FluxEeprom eeprom;
extern void readImu();
extern void calculateState();
extern float anglePitch, angleRoll, cosAlpha, gyroYaw;
extern int accX, accY, accZ, gyroX, gyroY, gyroZ;
extern int rcThrottle, rcRoll, rcPitch, rcYaw;

void remoteControlService() {
  Serial.println(F("-----------------------------------"));
  Serial.println(F("Fernsteuerung Test und Kalibration:"));
  Serial.println(F("Den Gashebel auf Null stellen (untere Position), dann alle Hebel loslassen."));
  Serial.println(F("Die Fernsteuerung so trimmen, dass fuer Gas ein Wert von 1000,"));
  Serial.println(F("fuer die drei anderen Kanaele 1500 empfangen wird."));
  Serial.println(F("Mit 'a' aktuelle Fernsteuerwerte anzeigen"));
  Serial.println(F("Mit 'e' beenden!"));

  while (Serial.available() == 0) ; // Wait for input
  char data = Serial.read();
  Serial.read();
  if (data == 'e') return;

  while (true) {
    Serial.print(F("Throttle = "));
    Serial.print(rcThrottle);
    Serial.print(F(", Yaw = "));
    Serial.print(rcYaw);
    Serial.print(F(", Pitch = "));
    Serial.print(rcPitch);
    Serial.print(F(", Roll = "));
    Serial.println(rcRoll);
    delay(1000);
    if (Serial.available() > 0) {
      Serial.read();
      Serial.read();
      return;
    }
  }
}

void imuTest() {
  while (true) {
    Serial.println(F("Mit 's' IMU starten oder 'f' fuer fertig eingeben!"));
    while (Serial.available() == 0) ; // Wait for input
    char data = Serial.read();
    Serial.read(); // Consume newline
    switch (data) {
      case 's': break;
      default: return;
    }
    Serial.println(F("Pitch, Roll (in grad), cos(Alpha), Yaw in grad/s"));
    unsigned long nextPrint = 0;
    unsigned long nextMeasure = 0;
    unsigned long now;
    while (true) {
      do {
        now = micros();
      }
      while (now < nextMeasure);
      nextMeasure = now + 1000 * LOOP_PERIOD;

      readImu();
      calculateState();

      if (now > nextPrint) {
        Serial.print(anglePitch, 2);
        Serial.print(F("  "));
        Serial.print(angleRoll, 2);
        Serial.print(F("  "));
        Serial.print(cosAlpha) ;
        Serial.print(F("  "));
        Serial.println(gyroYaw) ;
        nextPrint = now + 500000L;
      }
      if (Serial.available() > 0) {
        Serial.read();
        Serial.read(); // Consume newline
        break;
      }
    }
  }
}

// =============================================================
static long accXcal, accYcal, accZcal;
void doAverageAccelerationMeasurement() {
  const int REPEATS = 100;
  for (int i = 0; i < REPEATS; i++) readImu(); // warm up
  accXcal = accYcal = accZcal = 0;
  for (int i = 0; i < REPEATS; i++) {
    readImu();
    accXcal += accX;
    accYcal += accY;
    accZcal += accZ;
    delay(10);
  }
  accXcal /= REPEATS;
  accYcal /= REPEATS;
  accZcal /= REPEATS;
}

void horizontalCalibration() {
  while (true) {
    Serial.println(F("----------------------"));
    Serial.println(F("Horizontal-Kalibration"));
    Serial.println();
    Serial.println(F("Gespeicherte Kalibrationswerte:"));
    Serial.print(F("X = "));
    Serial.print(eeprom.accXhorizontal);
    Serial.print(F(", Y = "));
    Serial.print(eeprom.accYhorizontal);
    Serial.print(F(", Z = "));
    Serial.println(eeprom.accZhorizontal);

    while (true) {
      doAverageAccelerationMeasurement();
      Serial.println(F("Aktuelle Messwerte:"));
      Serial.print(F("X = "));
      Serial.print(accXcal);
      Serial.print(F(", Y = "));
      Serial.print(accYcal);
      Serial.print(F(", Z = "));
      Serial.println(accZcal);
      Serial.println();
      Serial.println(F("Den Quadcopter jetzt in eine waagerechte Position bringen!"));
      Serial.println(F("Mit 'k' Kalibration starten oder 'f' fuer fertig eingeben!"));

      while (Serial.available() == 0); // Wait for input
      char data = Serial.read();
      Serial.read(); // Consume newline
      switch (data) {
        case 'f': return;
        case 'k':
          doAverageAccelerationMeasurement();
          eeprom.accXhorizontal = accXcal;
          eeprom.accYhorizontal = accYcal;
          eeprom.accZhorizontal = accZcal;
          Serial.println(F("Werte wurden aufgenommen.\n"));
      }
    }
  }
}

void pidGainAdjust() {
  while (true) {
    Serial.println(F("----------------------------"));
    Serial.println(F("PID-Einstellung Lageregelung"));
    Serial.print(F("(1) pPitchRoll = "));
    Serial.print(eeprom.pPitchRoll);
    Serial.print(F(", (2) iPitchRoll = "));
    Serial.print(eeprom.iPitchRoll);
    Serial.print(F(", (3) dPitchRoll = "));
    Serial.println(eeprom.dPitchRoll);
    Serial.print(F("(4) pYaw = "));
    Serial.print(eeprom.pYaw);
    Serial.print(F(", (5) iYaw = "));
    Serial.print(eeprom.iYaw);
    Serial.print(F(", (6) dYaw = "));
    Serial.println(eeprom.dYaw);

    Serial.println(F("Bitte Einstellwert mit Ziffer auswaehlen oder 'f' fuer fertig!"));
    while (Serial.available() == 0) ; // Wait for input
    char data = Serial.read();
    Serial.read(); // Consume newline
    switch (data) {
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
        Serial.print(F("Neuen Wert ("));
        Serial.print(data);
        Serial.println(F(") mit Dezimal *PUNKT* eingeben:"));
        break;
      case 'f': return;
      default: continue;
    }
    while (Serial.available() == 0) ; // Wait for input
    float value = Serial.parseFloat();
    switch (data) {
      case '1': eeprom.pPitchRoll = value; break;
      case '2': eeprom.iPitchRoll = value; break;
      case '3': eeprom.dPitchRoll = value; break;
      case '4': eeprom.pYaw = value; break;
      case '5': eeprom.iYaw = value; break;
      case '6': eeprom.dYaw = value; break;
    }
  }
}

Tune balanceProps() {
  Serial.println(F("Auswahl Propeller:"));
  Serial.println(F("1 - Balancierung von Propeller FR"));
  Serial.println(F("2 - Balancierung von Propeller RR"));
  Serial.println(F("3 - Balancierung von Propeller RL"));
  Serial.println(F("4 - Balancierung von Propeller FL"));
  Serial.println(F("a - abbrechen"));
  Serial.println(F("Beachte: Nach erfolgtem Tuning muss der Arduino resettet werden!"));
  while (Serial.available() == 0) ; // Wait for input
  char data = Serial.read();
  Serial.read(); // Consume newline
  switch (data) {
    case '1':
      return Tune::BALANCE_PROP_FR;
    case '2':
      return Tune::BALANCE_PROP_RR;
    case '3':
      return Tune::BALANCE_PROP_RL;
    case '4':
      return Tune::BALANCE_PROP_FL;
    case 'a':
      return Tune::OFF;
    default:
      Serial.println(F("Eingabe nicht verstanden"));
      return Tune::OFF;
  }
}

Tune calibrateESCs() {
  Serial.println(F("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"));
  Serial.println(F("!!! Sicherheitshinweis: Bevor es weiter gehen kann, Propeller demontieren !!!"));
  Serial.println(F("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"));
  Serial.println(F("Jetzt Akku trennen, Fernsteuerung einschalten und den Gashebel auf Vollgas stellen"));
  Serial.println(F("Mit 'a' abbrechen, mit 'w' geht's weiter"));
  while (Serial.available() == 0) ; // Wait for input
  char data = Serial.read();
  switch (data) {
    case 'w':
      break;
    case 'a':
      return Tune::OFF;
    default:
      Serial.println(F("Eingabe nicht verstanden"));
      return Tune::OFF;
  }

  Serial.println();
  Serial.println(F("Jetzt Akku verbinden und warten bis alle ESCs sich beruhigt haben"));
  Serial.println(F("Danach den Gashebel schnell auf Null ziehen und wieder warten..."));
  Serial.println(F("Dann den Akku trennen und dieses Fenster schliessen - Fertig"));
  return Tune::CALIBRATE_ESCS;
}
