// Calculate state variables
#include "Arduino.h"
#include "Flux.hpp"
#include "Eeprom.h"

extern void readImu();
const int ACC_ONE_G = 4096; // Acceleromter value for 1g (= Gravitational acceleration on Earth's surface)

// Input data =====================================================================
extern int gyroX, gyroY, gyroZ, accX, accY, accZ, temperature;
extern FluxEeprom eeprom;

// Output data =====================================================================
// State variables
float anglePitch, angleRoll, angleYaw; // the current angles in degrees
float gyroPitch, gyroRoll, gyroYaw; // the current angle velocities in degrees per second
float cosAlpha; // Cosine of QC inclination

// Calibration =====================================================================
static int gyroXOffset;
static int gyroYOffset;
static int gyroZOffset;

void calibrateGyros() {
  const int REPEATS = 100;
  for (int i = 0; i < REPEATS; i++) readImu(); // warm up
  long gx, gy, gz;
  gx = gy = gz = 0;
  for (int i = 0; i < REPEATS; i++) {
    readImu();
    gx += gyroX;
    gy += gyroY;
    gz += gyroZ;
    delay(10);
  }
  gyroXOffset = gx / REPEATS;
  gyroYOffset = gy / REPEATS;
  gyroZOffset = gz / REPEATS;
}
// Calculation =====================================================================
static float xAccAvg, yAccAvg, zAccAvg;
void calculateState() {
  // Process accelerometer values
  float xComp = accX - eeprom.accXhorizontal;
  float yComp = accY - eeprom.accYhorizontal;
  float zComp = map(accZ, 0, eeprom.accZhorizontal, 0, ACC_ONE_G);
  float accRoll  = atan(yComp / zComp) * 57.3; // 57.3 = 360 / (2*Pi)
  float accPitch = atan(xComp / zComp) * 57.3; // conversion rad -> degree

  // Process gyroscope values, resulting unit is deg/sec (°/s)
  gyroRoll  =   (gyroX - gyroXOffset) / 65.5;
  gyroPitch = - (gyroY - gyroYOffset) / 65.5;
  gyroYaw   = - (gyroZ - gyroZOffset) / 65.5;

  { // Perform sensor fusion
  const float FUSION_TIME = 4.0;  // unit is seconds
  const float TAU = FUSION_TIME / STEP_TIME;
  const float ALPHA = TAU / (1 + TAU);
  const float BETA = 1 - ALPHA;
  anglePitch = ALPHA * (anglePitch + gyroPitch  * STEP_TIME) + BETA * accPitch;
  angleRoll  = ALPHA * (angleRoll  + gyroRoll   * STEP_TIME) + BETA * accRoll;
  }
  
  {   // Calculate declination after low pass filtering
  const float AVERAGING_TIME = 0.25; // unit is seconds
  const float TAU = AVERAGING_TIME / STEP_TIME;
  const float ALPHA = TAU / (1 + TAU);
  const float BETA = 1 - ALPHA;
  xAccAvg = ALPHA * xAccAvg + BETA * xComp;
  yAccAvg = ALPHA * yAccAvg + BETA * yComp;
  zAccAvg = ALPHA * zAccAvg + BETA * zComp;
  cosAlpha = abs(zAccAvg) / sqrt(xAccAvg * xAccAvg + yAccAvg * yAccAvg + zAccAvg * zAccAvg);
  }
}
// Vibration measurement for propeller balancing ===================================
static float average;
void measureVibration() {
  static long zaccX, zaccY, zaccZ;
  float noise = (accX - zaccX) * (accX - zaccX) + (accY - zaccY) * (accY - zaccY) + (accZ - zaccZ) * (accZ - zaccZ);
  average = 0.99 * average + 0.01 * noise;
  zaccX = accX;
  zaccY = accY;
  zaccZ = accZ;
}
float getVibration() {
  return average;
}
